"""File generated by TLObjects' generator. All changes will be ERASED"""
from ...tl.tlobject import TLObject
from typing import Optional, List, Union, TYPE_CHECKING
import os
import struct


class UseConfigSimpleRequest(TLObject):
    CONSTRUCTOR_ID = 0xf9b7b23d
    SUBCLASS_OF_ID = 0x29183ac4

    def __init__(self):
        super().__init__()
        self.result = None
        self.content_related = True

    def to_dict(self):
        return {
            '_': 'UseConfigSimpleRequest'
        }

    def __bytes__(self):
        return b''.join((
            b'=\xb2\xb7\xf9',
        ))

    @classmethod
    def from_reader(cls, reader):
        return cls()


class UseErrorRequest(TLObject):
    CONSTRUCTOR_ID = 0xee75af01
    SUBCLASS_OF_ID = 0x9c1c9375

    def __init__(self):
        super().__init__()
        self.result = None
        self.content_related = True

    def to_dict(self):
        return {
            '_': 'UseErrorRequest'
        }

    def __bytes__(self):
        return b''.join((
            b'\x01\xafu\xee',
        ))

    @classmethod
    def from_reader(cls, reader):
        return cls()

